@@ -127,6 +127,7 @@ import time
 import hmac
 import hashlib
 import re
+import httplib
 import urllib
 import urllib2
 import datetime
@@ -135,6 +136,10 @@ import mimetypes
 rfc822_fmt = '%a, %d %b %Y %H:%M:%S GMT'
 iso8601_fmt = '%Y-%m-%dT%H:%M:%S.000Z'
 
+__version__ = "0.4"
+
+__all__ = "S3File", "S3Bucket", "S3Error"
+
 def _amz_canonicalize(headers):
     r"""Canonicalize AMZ headers in that certain AWS way.
 
@@ -245,8 +250,8 @@ def name(o):
 
 class S3Error(Exception):
     def __init__(self, message, **kwds):
-        self.msg = message
-        self.extra = kwds.copy()
+        self.args = message, kwds.copy()
+        self.msg, self.extra = self.args
 
     def __str__(self):
         rv = self.msg
@@ -266,13 +271,17 @@ class S3Error(Exception):
             return self
         # The latter part of this clause is to avoid some weird bug in urllib2
         # and AWS which has it read as if chunked, and AWS gives empty reply.
-        self.data = data = fp.read()
-        begin, end = data.find("<Message>"), data.find("</Message>")
-        if min(begin, end) >= 0:
-            self.full_message = msg = data[begin + 9:end]
-            self.msg = msg[:100]
-            if self.msg != msg:
-                self.msg += "..."
+        try:
+            self.data = data = fp.read()
+        except (httplib.HTTPException, urllib2.URLError), e:
+            self.extra["read_error"] = e
+        else:
+            begin, end = data.find("<Message>"), data.find("</Message>")
+            if min(begin, end) >= 0:
+                self.full_message = msg = data[begin + 9:end]
+                self.msg = msg[:100]
+                if self.msg != msg:
+                    self.msg += "..."
         return self
 
 class StreamHTTPHandler(urllib2.HTTPHandler):
@@ -439,20 +448,19 @@ class S3Bucket(object):
         self.make_request("PUT", key=key, data=data, headers=headers).close()
 
     def delete(self, key):
+        # In <=py25, urllib2 raises an exception for HTTP 204, and later
+        # does not, so treat errors and non-errors as equals.
         try:
             resp = self.make_request("DELETE", key=key)
-            resp.close()
-        # Python 2.5: urllib2 raises an exception for 204.
         except S3Error, e:
-            if e.code == 204:
-                return True
-            elif e.code == 404:
-                raise KeyError(key)
-            raise
-        # Python 2.6: urllib2 does the right thing for 204.
+            resp = e
+        resp.close()
+        if 200 <= resp.code < 300:
+            return True
+        elif resp.code == 404:
+            raise KeyError(key)
         else:
-            if resp.code != 204:
-                raise KeyError(key)
+            raise S3Error.from_urllib(resp)
 
     def listdir(self, prefix=None, marker=None, limit=None, delimiter=None):
         """List contents of bucket.
